@@ -22,6 +22,12 @@
 #include <stdbool.h>
 #include <inttypes.h>
 
+#ifdef __GNUC__
+#define LIBHANGUL_DEPRECATED __attribute__((deprecated));
+#else
+#define LIBHANGUL_DEPRECATED
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -41,13 +47,10 @@ bool hangul_is_choseong_conjoinable(ucschar c);
 bool hangul_is_jungseong_conjoinable(ucschar c);
 bool hangul_is_jongseong_conjoinable(ucschar c);
 bool hangul_is_syllable(ucschar c);
-bool hangul_is_jaso(ucschar c);
 bool hangul_is_jamo(ucschar c);
+bool hangul_is_cjamo(ucschar c);
 
-ucschar hangul_jaso_to_jamo(ucschar ch);
-ucschar hangul_choseong_to_jamo(ucschar ch);
-ucschar hangul_jungseong_to_jamo(ucschar ch);
-ucschar hangul_jongseong_to_jamo(ucschar ch);
+ucschar hangul_jamo_to_cjamo(ucschar ch);
 
 ucschar hangul_choseong_to_jongseong(ucschar ch);
 ucschar hangul_jongseong_to_choseong(ucschar ch);
@@ -60,10 +63,10 @@ const ucschar* hangul_syllable_iterator_next(const ucschar* str,
 
 int     hangul_syllable_len(const ucschar* str, int max_len);
 
-ucschar hangul_jaso_to_syllable(ucschar choseong,
+ucschar hangul_jamo_to_syllable(ucschar choseong,
 				ucschar jungseong,
 				ucschar jongseong);
-void    hangul_syllable_to_jaso(ucschar syllable,
+void    hangul_syllable_to_jamo(ucschar syllable,
 				ucschar* choseong,
 				ucschar* jungseong,
 				ucschar* jongseong);
@@ -83,7 +86,8 @@ enum {
 
 enum {
     HANGUL_KEYBOARD_TYPE_JAMO,
-    HANGUL_KEYBOARD_TYPE_JASO
+    HANGUL_KEYBOARD_TYPE_JASO,
+    HANGUL_KEYBOARD_TYPE_ROMAJA
 };
 
 /* keyboard */
@@ -111,8 +115,6 @@ bool hangul_ic_has_choseong(HangulInputContext *hic);
 bool hangul_ic_has_jungseong(HangulInputContext *hic);
 bool hangul_ic_has_jongseong(HangulInputContext *hic);
 
-int  hangul_ic_dvorak_to_qwerty(int qwerty);
-
 void hangul_ic_set_output_mode(HangulInputContext *hic, int mode);
 void hangul_ic_set_keyboard(HangulInputContext *hic,
 			    const HangulKeyboard *keyboard);
@@ -138,9 +140,6 @@ HanjaList*   hanja_table_match_prefix(const HanjaTable* table, const char *key);
 HanjaList*   hanja_table_match_suffix(const HanjaTable* table, const char *key);
 void         hanja_table_delete(HanjaTable *table);
 
-int          hanja_table_txt_to_bin(const char* txtfilename,
-				    const char* binfilename);
-
 int          hanja_list_get_size(const HanjaList *list);
 const char*  hanja_list_get_key(const HanjaList *list);
 const Hanja* hanja_list_get_nth(const HanjaList *list, unsigned int n);
@@ -155,12 +154,25 @@ const char*  hanja_get_comment(const Hanja* hanja);
 
 
 /* deprecated */
+bool hangul_is_jaso(ucschar c) LIBHANGUL_DEPRECATED;
+ucschar hangul_jaso_to_jamo(ucschar ch) LIBHANGUL_DEPRECATED;
+ucschar hangul_jaso_to_syllable(ucschar choseong,
+				ucschar jungseong,
+				ucschar jongseong) LIBHANGUL_DEPRECATED;
+void    hangul_syllable_to_jaso(ucschar syllable,
+				ucschar* choseong,
+				ucschar* jungseong,
+				ucschar* jongseong) LIBHANGUL_DEPRECATED;
 typedef bool (*HangulICFilter) (ucschar*, ucschar, ucschar, ucschar, void*);
 void hangul_ic_set_filter(HangulInputContext *hic,
-			  HangulICFilter func, void *user_data);
+			  HangulICFilter func, void *user_data) LIBHANGUL_DEPRECATED;
+int  hangul_ic_dvorak_to_qwerty(int qwerty) LIBHANGUL_DEPRECATED;
+
 
 #ifdef __cplusplus
 }
 #endif
 
+#undef LIBHANGUL_DEPRECATED
+
 #endif /* libhangul_hangul_h */
